@@ -1215,9 +1215,7 @@ int32_t disasm(uint8_t *data, char *output, int outbufsize, int segsize,
 
         case 0x62:
         {
-            uint8_t evex_p0 = data[1] & 0x0f;
-            if (segsize == 64 ||
-                ((evex_p0 >= 0x01) && (evex_p0 <= 0x03))) {
+            if (segsize == 64 || ((data[1] & 0xc0) == 0xc0)) {
                 data++;        /* 62h EVEX prefix */
                 prefix.evex[0] = *data++;
                 prefix.evex[1] = *data++;
@@ -1684,7 +1682,7 @@ int32_t eatbyte(uint8_t *data, char *output, int outbufsize, int segsize)
         str = "ss";
         break;
     case 0x3E:
-        str = "ss";
+        str = "ds";
         break;
     case 0x26:
         str = "es";
