@@ -23,7 +23,7 @@ int vsnprintf(char *str, size_t size, const char *format, va_list ap)
     int rv, bytes;
 
     if (size > BUFFER_SIZE) {
-        nasm_error(ERR_PANIC|ERR_NOFILE,
+        nasm_panic(ERR_NOFILE,
                    "vsnprintf: size (%d) > BUFFER_SIZE (%d)",
                    size, BUFFER_SIZE);
         size = BUFFER_SIZE;
@@ -31,7 +31,7 @@ int vsnprintf(char *str, size_t size, const char *format, va_list ap)
 
     rv = vsprintf(snprintf_buffer, format, ap);
     if (rv >= BUFFER_SIZE)
-        nasm_error(ERR_PANIC|ERR_NOFILE, "vsnprintf buffer overflow");
+        nasm_panic(ERR_NOFILE, "vsnprintf buffer overflow");
 
     if (size > 0) {
         if ((size_t)rv < size-1)
