@@ -1,6 +1,6 @@
 /* ----------------------------------------------------------------------- *
  *
- *   Copyright 1996-2013 The NASM Authors - All Rights Reserved
+ *   Copyright 1996-2016 The NASM Authors - All Rights Reserved
  *   See the file AUTHORS included with the NASM distribution for
  *   the specific copyright holders.
  *
@@ -78,16 +78,18 @@ static int get_bits(char *value);
 static iflag_t get_cpu(char *cpu_str);
 static void parse_cmdline(int, char **);
 static void assemble_file(char *, StrList **);
+static bool is_suppressed_warning(int severity);
+static bool skip_this_pass(int severity);
 static void nasm_verror_gnu(int severity, const char *fmt, va_list args);
 static void nasm_verror_vc(int severity, const char *fmt, va_list args);
 static void nasm_verror_common(int severity, const char *fmt, va_list args);
-static bool is_suppressed_warning(int severity);
 static void usage(void);
 
-static int using_debug_info, opt_verbose_info;
+static bool using_debug_info, opt_verbose_info;
+static const char *debug_format;
+
 bool tasm_compatible_mode = false;
 int pass0, passn;
-int maxbits = 0;
 int globalrel = 0;
 int globalbnd = 0;
 
@@ -168,11 +170,12 @@ static const struct warning {
     {"lock", "lock prefix on unlockable instructions", true},
     {"hle", "invalid hle prefixes", true},
     {"bnd", "invalid bnd prefixes", true},
+    {"zext-reloc", "relocation zero-extended to match output format", true},
 };
 
 static bool want_usage;
 static bool terminate_after_phase;
-int user_nolist = 0;            /* fbk 9/2/00 */
+bool user_nolist = false;
 
 static char *quote_for_make(const char *str);
 
@@ -343,8 +346,6 @@ int main(int argc, char **argv)
     preproc = &nasmpp;
     operating_mode = OP_NORMAL;
 
-    seg_init();
-
     /* Define some macros dependent on the runtime, but not
        on the command line. */
     define_macros_early();
@@ -357,9 +358,21 @@ int main(int argc, char **argv)
         return 1;
     }
 
-    /* If debugging info is disabled, suppress any debug calls */
-    if (!using_debug_info)
-        ofmt->current_dfmt = &null_debug_form;
+    if (!using_debug_info) {
+        /* No debug info, redirect to the null backend (empty stubs) */
+        dfmt = &null_debug_form;
+    } else if (!debug_format) {
+        /* Default debug format for this backend */
+	dfmt = ofmt->default_dfmt;
+    } else {
+        dfmt = dfmt_find(ofmt, debug_format);
+        if (!dfmt) {
+            nasm_fatal(ERR_NOFILE | ERR_USAGE,
+                       "unrecognized debug format `%s' for"
+                       " output format `%s'",
+                       debug_format, ofmt->shortname);
+        }
+    }
 
     if (ofmt->stdmac)
         preproc->extra_stdmac(ofmt->stdmac);
@@ -379,7 +392,7 @@ int main(int argc, char **argv)
             if (depend_missing_ok)
                 preproc->include_path(NULL);    /* "assume generated" */
 
-            preproc->reset(inname, 0, &nasmlist, depend_ptr);
+            preproc->reset(inname, 0, depend_ptr);
             if (outname[0] == '\0')
                 ofmt->filename(inname, outname);
             ofile = NULL;
@@ -395,7 +408,7 @@ int main(int argc, char **argv)
             if (*outname) {
                 ofile = fopen(outname, "w");
                 if (!ofile)
-                    nasm_error(ERR_FATAL | ERR_NOFILE,
+                    nasm_fatal(ERR_NOFILE,
                                  "unable to open output file `%s'",
                                  outname);
             } else
@@ -404,8 +417,9 @@ int main(int argc, char **argv)
             location.known = false;
 
             /* pass = 1; */
-            preproc->reset(inname, 3, &nasmlist, depend_ptr);
-            memcpy(warning_on, warning_on_global, (ERR_WARN_MAX+1) * sizeof(bool));
+            preproc->reset(inname, 3, depend_ptr);
+            memcpy(warning_on, warning_on_global,
+		   (ERR_WARN_MAX+1) * sizeof(bool));
 
             while ((line = preproc->getline())) {
                 /*
@@ -448,7 +462,7 @@ int main(int argc, char **argv)
 
         ofile = fopen(outname, (ofmt->flags & OFMT_TEXT) ? "w" : "wb");
         if (!ofile)
-            nasm_error(ERR_FATAL | ERR_NOFILE,
+            nasm_fatal(ERR_NOFILE,
                        "unable to open output file `%s'", outname);
 
         /*
@@ -459,13 +473,12 @@ int main(int argc, char **argv)
         init_labels();
 
         ofmt->init();
-        dfmt = ofmt->current_dfmt;
         dfmt->init();
 
         assemble_file(inname, depend_ptr);
 
         if (!terminate_after_phase) {
-            ofmt->cleanup(using_debug_info);
+            ofmt->cleanup();
             cleanup_labels();
             fflush(ofile);
             if (ferror(ofile))
@@ -521,7 +534,7 @@ static void copy_filename(char *dst, const char *src)
     size_t len = strlen(src);
 
     if (len >= (size_t)FILENAME_MAX) {
-        nasm_error(ERR_FATAL | ERR_NOFILE, "file name too long");
+        nasm_fatal(ERR_NOFILE, "file name too long");
         return;
     }
     strncpy(dst, src, FILENAME_MAX);
@@ -614,8 +627,10 @@ struct textargs {
     int value;
 };
 
-#define OPT_PREFIX 0
-#define OPT_POSTFIX 1
+enum text_options {
+    OPT_PREFIX,
+    OPT_POSTFIX
+};
 struct textargs textopts[] = {
     {"prefix", OPT_PREFIX},
     {"postfix", OPT_POSTFIX},
@@ -659,7 +674,7 @@ static bool process_arg(char *p, char *q)
         case 'f':       /* output format */
             ofmt = ofmt_find(param, &ofmt_alias);
             if (!ofmt) {
-                nasm_error(ERR_FATAL | ERR_NOFILE | ERR_USAGE,
+                nasm_fatal(ERR_NOFILE | ERR_USAGE,
                            "unrecognised output format `%s' - "
                            "use -hf for a list", param);
             }
@@ -699,7 +714,7 @@ static bool process_arg(char *p, char *q)
                         break;
 
                     default:
-                        nasm_error(ERR_FATAL,
+                        nasm_fatal(0,
                                    "unknown optimization option -O%c\n",
                                    *param);
                         break;
@@ -740,14 +755,8 @@ static bool process_arg(char *p, char *q)
             break;
 
         case 'F':       /* specify debug format */
-            ofmt->current_dfmt = dfmt_find(ofmt, param);
-            if (!ofmt->current_dfmt) {
-                nasm_error(ERR_FATAL | ERR_NOFILE | ERR_USAGE,
-                           "unrecognized debug format `%s' for"
-                           " output format `%s'",
-                           param, ofmt->shortname);
-            }
             using_debug_info = true;
+            debug_format = param;
             break;
 
         case 'X':       /* specify error reporting format */
@@ -756,13 +765,15 @@ static bool process_arg(char *p, char *q)
             else if (nasm_stricmp("gnu", param) == 0)
                 nasm_set_verror(nasm_verror_gnu);
             else
-                nasm_error(ERR_FATAL | ERR_NOFILE | ERR_USAGE,
+                nasm_fatal(ERR_NOFILE | ERR_USAGE,
                            "unrecognized error reporting format `%s'",
                            param);
             break;
 
         case 'g':
             using_debug_info = true;
+            if (p[2])
+                debug_format = nasm_skip_spaces(p + 2);
             break;
 
         case 'h':
@@ -771,8 +782,7 @@ static bool process_arg(char *p, char *q)
                  "[-l listfile]\n"
                  "            [options...] [--] filename\n"
                  "    or nasm -v (or --v) for version info\n\n"
-                 "    -t          assemble in SciTech TASM compatible mode\n"
-                 "    -g          generate debug information in selected format\n");
+                 "    -t          assemble in SciTech TASM compatible mode\n");
             printf
                 ("    -E (or -e)  preprocess only (writes output to stdout by default)\n"
                  "    -a          don't preprocess (assemble only)\n"
@@ -785,7 +795,9 @@ static bool process_arg(char *p, char *q)
                  "    -MP         emit phony target\n\n"
                  "    -Z<file>    redirect error messages to file\n"
                  "    -s          redirect error messages to stdout\n\n"
+                 "    -g          generate debugging information\n\n"
                  "    -F format   select a debugging format\n\n"
+                 "    -gformat    same as -g -F format\n\n"
                  "    -o outfile  write output to an outfile\n\n"
                  "    -f format   select an output format\n\n"
                  "    -l listfile write listing to a listfile\n\n"
@@ -804,7 +816,7 @@ static bool process_arg(char *p, char *q)
                  "    -h          show invocation summary and exit\n\n"
                  "--prefix,--postfix\n"
                  "  this options prepend or append the given argument to all\n"
-                 "  extern and global variables\n\n"
+                 "  extern and global variables\n"
                  "Warnings:\n");
             for (i = 0; i <= ERR_WARN_MAX; i++)
                 printf("    %-23s %s (default %s)\n",
@@ -959,18 +971,21 @@ set_warning:
                             advance = 1, param = q;
                         }
 
-                        if (s == OPT_PREFIX) {
-                            strncpy(lprefix, param, PREFIX_MAX - 1);
-                            lprefix[PREFIX_MAX - 1] = 0;
+                        switch (s) {
+                        case OPT_PREFIX:
+                            strlcpy(lprefix, param, PREFIX_MAX);
                             break;
-                        }
-                        if (s == OPT_POSTFIX) {
-                            strncpy(lpostfix, param, POSTFIX_MAX - 1);
-                            lpostfix[POSTFIX_MAX - 1] = 0;
+                        case OPT_POSTFIX:
+                            strlcpy(lpostfix, param, POSTFIX_MAX);
+                            break;
+                        default:
+                            nasm_panic(ERR_NOFILE,
+                                       "internal error");
                             break;
                         }
                         break;
                     }
+
                 default:
                     {
                         nasm_error(ERR_NONFATAL | ERR_NOFILE | ERR_USAGE,
@@ -1167,7 +1182,7 @@ static void parse_cmdline(int argc, char **argv)
              !strcmp(inname, outname)   ||
              !strcmp(inname, listname)  ||
              (depend_file && !strcmp(inname, depend_file)))
-        nasm_error(ERR_FATAL | ERR_NOFILE | ERR_USAGE,
+        nasm_fatal(ERR_NOFILE | ERR_USAGE,
                    "file `%s' is both input and output file",
                    inname);
 
@@ -1175,7 +1190,7 @@ static void parse_cmdline(int argc, char **argv)
         error_file = fopen(errname, "w");
         if (!error_file) {
             error_file = stderr;        /* Revert to default! */
-            nasm_error(ERR_FATAL | ERR_NOFILE | ERR_USAGE,
+            nasm_fatal(ERR_NOFILE | ERR_USAGE,
                        "cannot open file `%s' for error messages",
                        errname);
         }
@@ -1197,8 +1212,7 @@ static void assemble_file(char *fname, StrList **depend_ptr)
     int pass_max;
 
     if (cmd_sb == 32 && iflag_ffs(&cmd_cpu) < IF_386)
-        nasm_error(ERR_FATAL, "command line: "
-                     "32-bit segment size requires a higher cpu");
+	nasm_fatal(0, "command line: 32-bit segment size requires a higher cpu");
 
     pass_max = prev_offset_changed = (INT_MAX >> 1) + 2; /* Almost unlimited */
     for (passn = 1; pass0 <= 2; passn++) {
@@ -1214,8 +1228,7 @@ static void assemble_file(char *fname, StrList **depend_ptr)
         globalbits = sb = cmd_sb;   /* set 'bits' to command line default */
         cpu = cmd_cpu;
         if (pass0 == 2) {
-            if (*listname)
-                nasmlist.init(listname, nasm_error);
+	    lfmt->init(listname);
         }
         in_abs_seg = false;
         global_offset_changed = 0;  /* set by redefine_label */
@@ -1227,8 +1240,7 @@ static void assemble_file(char *fname, StrList **depend_ptr)
             raa_free(offsets);
             offsets = raa_init();
         }
-        preproc->reset(fname, pass1, &nasmlist,
-                       pass1 == 2 ? depend_ptr : NULL);
+        preproc->reset(fname, pass1, pass1 == 2 ? depend_ptr : NULL);
         memcpy(warning_on, warning_on_global, (ERR_WARN_MAX+1) * sizeof(bool));
 
         globallineno = 0;
@@ -1267,7 +1279,7 @@ static void assemble_file(char *fname, StrList **depend_ptr)
                         stdscan_reset();
                         stdscan_set(value);
                         tokval.t_type = TOKEN_INVALID;
-                        e = evaluate(stdscan, NULL, &tokval, NULL, pass2, nasm_error, NULL);
+                        e = evaluate(stdscan, NULL, &tokval, NULL, pass2, NULL);
                         if (e) {
                             unsigned int align = (unsigned int)e->value;
                             if ((uint64_t)e->value > 0x7fffffff) {
@@ -1276,15 +1288,17 @@ static void assemble_file(char *fname, StrList **depend_ptr)
                                  * ofmt should have some 'check' method which
                                  * would report segment alignment bounds.
                                  */
-                                nasm_error(ERR_FATAL,
+                                nasm_fatal(0,
                                            "incorrect segment alignment `%s'", value);
                             } else if (!is_power2(align)) {
                                 nasm_error(ERR_NONFATAL,
                                            "segment alignment `%s' is not power of two",
                                             value);
                             }
+
                             /* callee should be able to handle all details */
-                            ofmt->sectalign(location.segment, align);
+			    if (location.segment != NO_SEG)
+			      ofmt->sectalign(location.segment, align);
                         }
                     }
                     break;
@@ -1423,8 +1437,7 @@ static void assemble_file(char *fname, StrList **depend_ptr)
                     stdscan_reset();
                     stdscan_set(value);
                     tokval.t_type = TOKEN_INVALID;
-                    e = evaluate(stdscan, NULL, &tokval, NULL, pass2,
-                                 nasm_error, NULL);
+                    e = evaluate(stdscan, NULL, &tokval, NULL, pass2, NULL);
                     if (e) {
                         if (!is_reloc(e))
                             nasm_error(pass0 ==
@@ -1438,7 +1451,7 @@ static void assemble_file(char *fname, StrList **depend_ptr)
                     } else if (passn == 1)
                         abs_offset = 0x100;     /* don't go near zero in case of / */
                     else
-                        nasm_error(ERR_PANIC, "invalid ABSOLUTE address "
+                        nasm_panic(0, "invalid ABSOLUTE address "
                                      "in pass two");
                     in_abs_seg = true;
                     location.segment = NO_SEG;
@@ -1671,7 +1684,7 @@ static void assemble_file(char *fname, StrList **depend_ptr)
                     if (pass1 == 1) {
 
                         int64_t l = insn_size(location.segment, offs, sb, cpu,
-                                           &output_ins, nasm_error);
+					      &output_ins);
 
                         /* if (using_debug_info)  && output_ins.opcode != -1) */
                         if (using_debug_info)
@@ -1750,8 +1763,7 @@ static void assemble_file(char *fname, StrList **depend_ptr)
 
                     } else {
                         offs += assemble(location.segment, offs, sb, cpu,
-                                         &output_ins, ofmt, nasm_error,
-                                         &nasmlist);
+                                         &output_ins);
                         set_curr_offs(offs);
 
                     }
@@ -1796,7 +1808,7 @@ static void assemble_file(char *fname, StrList **depend_ptr)
     }
 
     preproc->cleanup(0);
-    nasmlist.cleanup();
+    lfmt->cleanup();
     if (!terminate_after_phase && opt_verbose_info) {
         /*  -On and -Ov switches */
         fprintf(stdout, "info: assembly required 1+%d+1 passes\n", passn-3);
@@ -1868,11 +1880,13 @@ static void nasm_verror_gnu(int severity, const char *fmt, va_list ap)
     if (!(severity & ERR_NOFILE))
         src_get(&lineno, &currentfile);
 
-    if (currentfile) {
-        fprintf(error_file, "%s:%"PRId32": ", currentfile, lineno);
-        nasm_free(currentfile);
-    } else {
-        fputs("nasm: ", error_file);
+    if (!skip_this_pass(severity)) {
+	if (currentfile) {
+	    fprintf(error_file, "%s:%"PRId32": ", currentfile, lineno);
+	    nasm_free(currentfile);
+	} else {
+	    fputs("nasm: ", error_file);
+	}
     }
 
     nasm_verror_common(severity, fmt, ap);
@@ -1904,11 +1918,13 @@ static void nasm_verror_vc(int severity, const char *fmt, va_list ap)
     if (!(severity & ERR_NOFILE))
         src_get(&lineno, &currentfile);
 
-    if (currentfile) {
-        fprintf(error_file, "%s(%"PRId32") : ", currentfile, lineno);
-        nasm_free(currentfile);
-    } else {
-        fputs("nasm: ", error_file);
+    if (!skip_this_pass(severity)) {
+        if (currentfile) {
+	    fprintf(error_file, "%s(%"PRId32") : ", currentfile, lineno);
+	    nasm_free(currentfile);
+	} else {
+	    fputs("nasm: ", error_file);
+	}
     }
 
     nasm_verror_common(severity, fmt, ap);
@@ -1928,11 +1944,6 @@ static bool is_suppressed_warning(int severity)
     if ((severity & ERR_MASK) != ERR_WARNING)
         return false;
 
-    /* See if it's a pass-one only warning and we're not in pass one. */
-    if (((severity & ERR_PASS1) && pass0 != 1) ||
-        ((severity & ERR_PASS2) && pass0 != 2))
-        return true;
-
     /* Might be a warning but suppresed explicitly */
     if (severity & ERR_WARN_MASK)
         return !warning_on[WARN_IDX(severity)];
@@ -1940,6 +1951,19 @@ static bool is_suppressed_warning(int severity)
         return false;
 }
 
+static bool skip_this_pass(int severity)
+{
+    /* See if it's a pass-one only warning and we're not in pass one. */
+    if ((severity & ERR_MASK) > ERR_WARNING)
+	return false;
+
+    if (((severity & ERR_PASS1) && pass0 != 1) ||
+        ((severity & ERR_PASS2) && pass0 != 2))
+        return true;
+
+    return false;
+}
+
 /**
  * common error reporting
  * This is the common back end of the error reporting schemes currently
@@ -1978,10 +2002,15 @@ static void nasm_verror_common(int severity, const char *fmt, va_list args)
 
     vsnprintf(msg, sizeof msg, fmt, args);
 
-    fprintf(error_file, "%s%s\n", pfx, msg);
+    if (!skip_this_pass(severity))
+	fprintf(error_file, "%s%s\n", pfx, msg);
 
-    if (*listname)
-        nasmlist.error(severity, pfx, msg);
+    /*
+     * Don't suppress this with skip_this_pass(), or we don't get
+     * preprocessor warnings in the list file
+     */
+    if ((severity & ERR_MASK) >= ERR_WARNING)
+	lfmt->error(severity, pfx, msg);
 
     if (severity & ERR_USAGE)
         want_usage = true;
@@ -2011,6 +2040,11 @@ static void nasm_verror_common(int severity, const char *fmt, va_list args)
     case ERR_PANIC:
         fflush(NULL);
         /* abort(); */          /* halt, catch fire, and dump core */
+        if (ofile) {
+            fclose(ofile);
+            remove(outname);
+            ofile = NULL;
+        }
         exit(3);
         break;
     }
@@ -2088,12 +2122,6 @@ static int get_bits(char *value)
                          "cannot specify 64-bit segment on processor below an x86-64");
             i = 16;
         }
-        if (i != maxbits) {
-            nasm_error(ERR_NONFATAL,
-                         "%s output format does not support 64-bit code",
-                         ofmt->shortname);
-            i = 16;
-        }
     } else {
         nasm_error(pass0 < 2 ? ERR_NONFATAL : ERR_FATAL,
                      "`%s' is not a valid segment size; must be 16, 32 or 64",
