@@ -1,6 +1,6 @@
 /* ----------------------------------------------------------------------- *
  *   
- *   Copyright 1996-2014 The NASM Authors - All Rights Reserved
+ *   Copyright 1996-2016 The NASM Authors - All Rights Reserved
  *   See the file AUTHORS included with the NASM distribution for
  *   the specific copyright holders.
  *
@@ -49,7 +49,7 @@
 #include "insns.h"
 
 int globalbits = 0;    /* defined in nasm.h, works better here for ASM+DISASM */
-static vefunc nasm_verror;	/* Global error handling function */
+vefunc nasm_verror;    /* Global error handling function */
 
 /* Uninitialized -> all zero by C spec */
 const uint8_t zero_buffer[ZERO_BUF_SIZE];
@@ -69,11 +69,6 @@ void tolower_init(void)
 	nasm_tolower_tab[i] = tolower(i);
 }
 
-void nasm_set_verror(vefunc ve)
-{
-    nasm_verror = ve;
-}
-
 void nasm_error(int severity, const char *fmt, ...)
 {
     va_list ap;
@@ -83,11 +78,34 @@ void nasm_error(int severity, const char *fmt, ...)
     va_end(ap);
 }
 
+no_return nasm_fatal(int flags, const char *fmt, ...)
+{
+    va_list ap;
+
+    va_start(ap, fmt);
+    nasm_verror(flags | ERR_FATAL, fmt, ap);
+    abort();			/* We should never get here */
+}
+
+no_return nasm_panic(int flags, const char *fmt, ...)
+{
+    va_list ap;
+
+    va_start(ap, fmt);
+    nasm_verror(flags | ERR_PANIC, fmt, ap);
+    abort();			/* We should never get here */
+}
+
+no_return nasm_panic_from_macro(const char *file, int line)
+{
+    nasm_panic(ERR_NOFILE, "Internal error at %s:%d\n", file, line);
+}
+
 void *nasm_malloc(size_t size)
 {
     void *p = malloc(size);
     if (!p)
-        nasm_error(ERR_FATAL | ERR_NOFILE, "out of memory");
+        nasm_fatal(ERR_NOFILE, "out of memory");
     return p;
 }
 
@@ -95,7 +113,7 @@ void *nasm_zalloc(size_t size)
 {
     void *p = calloc(size, 1);
     if (!p)
-        nasm_error(ERR_FATAL | ERR_NOFILE, "out of memory");
+        nasm_fatal(ERR_NOFILE, "out of memory");
     return p;
 }
 
@@ -103,7 +121,7 @@ void *nasm_realloc(void *q, size_t size)
 {
     void *p = q ? realloc(q, size) : malloc(size);
     if (!p)
-        nasm_error(ERR_FATAL | ERR_NOFILE, "out of memory");
+        nasm_fatal(ERR_NOFILE, "out of memory");
     return p;
 }
 
@@ -120,7 +138,7 @@ char *nasm_strdup(const char *s)
 
     p = malloc(size);
     if (!p)
-        nasm_error(ERR_FATAL | ERR_NOFILE, "out of memory");
+        nasm_fatal(ERR_NOFILE, "out of memory");
     strcpy(p, s);
     return p;
 }
@@ -132,7 +150,7 @@ char *nasm_strndup(const char *s, size_t len)
 
     p = malloc(size);
     if (!p)
-        nasm_error(ERR_FATAL | ERR_NOFILE, "out of memory");
+        nasm_fatal(ERR_NOFILE, "out of memory");
     strncpy(p, s, len);
     p[len] = '\0';
     return p;
@@ -140,7 +158,7 @@ char *nasm_strndup(const char *s, size_t len)
 
 no_return nasm_assert_failed(const char *file, int line, const char *msg)
 {
-    nasm_error(ERR_FATAL, "assertion %s failed at %s:%d", msg, file, line);
+    nasm_fatal(0, "assertion %s failed at %s:%d", msg, file, line);
     exit(1);
 }
 
@@ -148,7 +166,7 @@ void nasm_write(const void *ptr, size_t size, FILE *f)
 {
     size_t n = fwrite(ptr, 1, size, f);
     if (n != size)
-        nasm_error(ERR_FATAL, "unable to write output: %s", strerror(errno));
+        nasm_fatal(0, "unable to write output: %s", strerror(errno));
 }
 
 #ifndef nasm_stricmp
@@ -377,16 +395,14 @@ int64_t readstrnum(char *str, int length, bool *warn)
     return charconst;
 }
 
-static int32_t next_seg;
-
-void seg_init(void)
-{
-    next_seg = 0;
-}
-
 int32_t seg_alloc(void)
 {
-    return (next_seg += 2) - 2;
+    static int32_t next_seg = 0;
+    int32_t this_seg = next_seg;
+
+    next_seg += 2;
+
+    return this_seg;
 }
 
 #ifdef WORDS_LITTLEENDIAN
