@@ -138,12 +138,10 @@ static void as86_init(void)
     as86_add_string(as86_module);
 }
 
-static void as86_cleanup(int debuginfo)
+static void as86_cleanup(void)
 {
     struct Piece *p;
 
-    (void)debuginfo;
-
     as86_write();
     saa_free(stext.data);
     while (stext.head) {
@@ -330,6 +328,8 @@ static void as86_out(int32_t segto, const void *data,
         return;
     }
 
+    memset(mydata, 0, sizeof(mydata));
+
     if (type == OUT_RESERVE) {
         if (s) {
             nasm_error(ERR_WARNING, "uninitialized space declared in"
@@ -341,11 +341,11 @@ static void as86_out(int32_t segto, const void *data,
             bsslen += size;
     } else if (type == OUT_RAWDATA) {
         if (segment != NO_SEG)
-            nasm_error(ERR_PANIC, "OUT_RAWDATA with other than NO_SEG");
+            nasm_panic(0, "OUT_RAWDATA with other than NO_SEG");
         as86_sect_write(s, data, size);
         as86_add_piece(s, 0, 0L, 0L, size, 0);
     } else if (type == OUT_ADDRESS) {
-        int asize = abs(size);
+        int asize = abs((int)size);
         if (segment != NO_SEG) {
             if (segment % 2) {
                 nasm_error(ERR_NONFATAL, "as86 format does not support"
@@ -362,7 +362,7 @@ static void as86_out(int32_t segto, const void *data,
         }
     } else if (type == OUT_REL2ADR) {
         if (segment == segto)
-            nasm_error(ERR_PANIC, "intra-segment OUT_REL2ADR");
+            nasm_panic(0, "intra-segment OUT_REL2ADR");
         if (segment != NO_SEG) {
             if (segment % 2) {
                 nasm_error(ERR_NONFATAL, "as86 format does not support"
@@ -375,7 +375,7 @@ static void as86_out(int32_t segto, const void *data,
         }
     } else if (type == OUT_REL4ADR) {
         if (segment == segto)
-            nasm_error(ERR_PANIC, "intra-segment OUT_REL4ADR");
+            nasm_panic(0, "intra-segment OUT_REL4ADR");
         if (segment != NO_SEG) {
             if (segment % 2) {
                 nasm_error(ERR_NONFATAL, "as86 format does not support"
@@ -516,7 +516,7 @@ static void as86_set_rsize(int size)
             fputc(0x03, ofile);
             break;
         default:
-            nasm_error(ERR_PANIC, "bizarre relocation size %d", size);
+            nasm_panic(0, "bizarre relocation size %d", size);
 	    break;
         }
     }
@@ -631,6 +631,7 @@ struct ofmt of_as86 = {
     "Linux as86 (bin86 version 0.3) object files",
     "as86",
     0,
+    32,
     null_debug_arr,
     &null_debug_form,
     as86_stdmac,
