@@ -1,6 +1,6 @@
 /* ----------------------------------------------------------------------- *
  *
- *   Copyright 1996-2013 The NASM Authors - All Rights Reserved
+ *   Copyright 1996-2016 The NASM Authors - All Rights Reserved
  *   See the file AUTHORS included with the NASM distribution for
  *   the specific copyright holders.
  *
@@ -161,8 +161,6 @@ static struct elf_symbol *lastsym;
 
 /* common debugging routines */
 static void debug64_typevalue(int32_t);
-static void debug64_deflabel(char *, int32_t, int64_t, int, char *);
-static void debug64_directive(const char *, const char *);
 
 /* stabs debugging routines */
 static void stabs64_linenum(const char *filename, int32_t linenumber, int32_t);
@@ -191,7 +189,6 @@ static int32_t elf_gottpoff_sect;
 
 static void elf_init(void)
 {
-    maxbits = 64;
     sects = NULL;
     nsects = sectlen = 0;
     syms = saa_init((int32_t)sizeof(struct elf_symbol));
@@ -224,13 +221,11 @@ static void elf_init(void)
 
 }
 
-static void elf_cleanup(int debuginfo)
+static void elf_cleanup(void)
 {
     struct elf_reloc *r;
     int i;
 
-    (void)debuginfo;
-
     elf_write();
     for (i = 0; i < nsects; i++) {
         if (sects[i]->type != SHT_NOBITS)
@@ -247,9 +242,7 @@ static void elf_cleanup(int debuginfo)
     saa_free(syms);
     raa_free(bsym);
     saa_free(strs);
-    if (of_elf64.current_dfmt) {
-        of_elf64.current_dfmt->cleanup();
-    }
+    dfmt->cleanup();
 }
 
 /* add entry to the elf .shstrtab section */
@@ -389,7 +382,7 @@ static void elf_deflabel(char *name, int32_t segment, int64_t offset,
                 stdscan_reset();
                 stdscan_set(p);
                 tokval.t_type = TOKEN_INVALID;
-                e = evaluate(stdscan, NULL, &tokval, NULL, 1, nasm_error, NULL);
+                e = evaluate(stdscan, NULL, &tokval, NULL, 1, NULL);
                 if (e) {
                     if (!is_simple(e))
                         nasm_error(ERR_NONFATAL, "cannot use relocatable"
@@ -428,7 +421,7 @@ static void elf_deflabel(char *name, int32_t segment, int64_t offset,
             /* we have to be sure at least text section is there */
             int tempint;
             if (segment != elf_section_names(".text", 2, &tempint))
-                nasm_error(ERR_PANIC, "strange segment conditions in ELF driver");
+                nasm_panic(0, "strange segment conditions in ELF driver");
         }
         for (i = 0; i < nsects; i++) {
             if (segment == sects[i]->index) {
@@ -531,8 +524,7 @@ static void elf_deflabel(char *name, int32_t segment, int64_t offset,
                     stdscan_reset();
                     stdscan_set(special + n);
                     tokval.t_type = TOKEN_INVALID;
-                    e = evaluate(stdscan, NULL, &tokval, &fwd, 0, nasm_error,
-                                 NULL);
+                    e = evaluate(stdscan, NULL, &tokval, &fwd, 0, NULL);
                     if (fwd) {
                         sym->nextfwd = fwds;
                         fwds = sym;
@@ -703,7 +695,7 @@ static void elf_out(int32_t segto, const void *data,
     if (!s) {
         int tempint;            /* ignored */
         if (segto != elf_section_names(".text", 2, &tempint))
-            nasm_error(ERR_PANIC, "strange segment conditions in ELF driver");
+            nasm_panic(0, "strange segment conditions in ELF driver");
         else {
             s = sects[nsects - 1];
             i = nsects - 1;
@@ -711,13 +703,11 @@ static void elf_out(int32_t segto, const void *data,
     }
 
     /* again some stabs debugging stuff */
-    if (of_elf64.current_dfmt) {
-        sinfo.offset = s->len;
-        sinfo.section = i;
-        sinfo.segto = segto;
-        sinfo.name = s->name;
-        of_elf64.current_dfmt->debug_output(TY_DEBUGSYMLIN, &sinfo);
-    }
+    sinfo.offset = s->len;
+    sinfo.section = i;
+    sinfo.segto = segto;
+    sinfo.name = s->name;
+    dfmt->debug_output(TY_DEBUGSYMLIN, &sinfo);
     /* end of debugging stuff */
 
     if (s->type == SHT_NOBITS && type != OUT_RESERVE) {
@@ -739,14 +729,14 @@ static void elf_out(int32_t segto, const void *data,
 
     case OUT_RAWDATA:
         if (segment != NO_SEG)
-            nasm_error(ERR_PANIC, "OUT_RAWDATA with other than NO_SEG");
+            nasm_panic(0, "OUT_RAWDATA with other than NO_SEG");
         elf_sect_write(s, data, size);
 	break;
 
     case OUT_ADDRESS:
     {
         int isize = (int)size;
-        int asize = abs(size);
+        int asize = abs((int)size);
 
         addr = *(int64_t *)data;
         if (segment == NO_SEG) {
@@ -776,7 +766,7 @@ static void elf_out(int32_t segto, const void *data,
                     elf_add_reloc(s, segment, addr, R_X86_64_64);
                     break;
                 default:
-                    nasm_error(ERR_PANIC, "internal error elf64-hpa-871");
+                    nasm_panic(0, "internal error elf64-hpa-871");
                     break;
                 }
                 addr = 0;
@@ -844,7 +834,7 @@ static void elf_out(int32_t segto, const void *data,
                     addr = 0;
                     break;
                 default:
-                    nasm_error(ERR_PANIC, "internal error elf64-hpa-903");
+                    nasm_panic(0, "internal error elf64-hpa-903");
                     break;
                 }
             } else if (wrt == elf_plt_sect + 1) {
@@ -872,7 +862,7 @@ static void elf_out(int32_t segto, const void *data,
     rel12adr:
         addr = *(int64_t *)data - size;
         if (segment == segto)
-            nasm_error(ERR_PANIC, "intra-segment OUT_REL1ADR");
+            nasm_panic(0, "intra-segment OUT_REL1ADR");
         if (segment == NO_SEG) {
             /* Do nothing */
         } else if (segment % 2) {
@@ -893,7 +883,7 @@ static void elf_out(int32_t segto, const void *data,
     case OUT_REL4ADR:
         addr = *(int64_t *)data - size;
         if (segment == segto)
-            nasm_error(ERR_PANIC, "intra-segment OUT_REL4ADR");
+            nasm_panic(0, "intra-segment OUT_REL4ADR");
         if (segment == NO_SEG) {
             /* Do nothing */
         } else if (segment % 2) {
@@ -931,7 +921,7 @@ static void elf_out(int32_t segto, const void *data,
     case OUT_REL8ADR:
         addr = *(int64_t *)data - size;
         if (segment == segto)
-            nasm_error(ERR_PANIC, "intra-segment OUT_REL8ADR");
+            nasm_panic(0, "intra-segment OUT_REL8ADR");
         if (segment == NO_SEG) {
             /* Do nothing */
         } else if (segment % 2) {
@@ -979,9 +969,9 @@ static void elf_write(void)
      * relocation sections for the user sections.
      */
     nsections = sec_numspecial + 1;
-    if (of_elf64.current_dfmt == &df_stabs)
+    if (dfmt == &df_stabs)
         nsections += 3;
-    else if (of_elf64.current_dfmt == &df_dwarf)
+    else if (dfmt == &df_dwarf)
         nsections += 10;
 
     add_sectname("", ".shstrtab");
@@ -995,14 +985,14 @@ static void elf_write(void)
         }
     }
 
-    if (of_elf64.current_dfmt == &df_stabs) {
+    if (dfmt == &df_stabs) {
         /* in case the debug information is wanted, just add these three sections... */
         add_sectname("", ".stab");
         add_sectname("", ".stabstr");
         add_sectname(".rel", ".stab");
     }
 
-    else if (of_elf64.current_dfmt == &df_dwarf) {
+    else if (dfmt == &df_dwarf) {
         /* the dwarf debug standard specifies the following ten sections,
            not all of which are currently implemented,
            although all of them are defined. */
@@ -1084,7 +1074,7 @@ static void elf_write(void)
 
     /* .symtab */
     elf_section_header(p - shstrtab, SHT_SYMTAB, 0, symtab, true,
-                       symtablen, sec_strtab, symtablocal, 4, 24);
+                       symtablen, sec_strtab, symtablocal, 8, 24);
     p += strlen(p) + 1;
 
     /* .strtab */
@@ -1096,11 +1086,11 @@ static void elf_write(void)
     for (i = 0; i < nsects; i++)
         if (sects[i]->head) {
             elf_section_header(p - shstrtab, SHT_RELA, 0, sects[i]->rel, true,
-                               sects[i]->rellen, sec_symtab, i + 1, 4, 24);
+                               sects[i]->rellen, sec_symtab, i + 1, 8, 24);
             p += strlen(p) + 1;
         }
 
-    if (of_elf64.current_dfmt == &df_stabs) {
+    if (dfmt == &df_stabs) {
         /* for debugging information, create the last three sections
            which are the .stab , .stabstr and .rel.stab sections respectively */
 
@@ -1121,7 +1111,7 @@ static void elf_write(void)
                                stabrellen, symtabsection, sec_stab, 4, 16);
             p += strlen(p) + 1;
         }
-    } else if (of_elf64.current_dfmt == &df_dwarf) {
+    } else if (dfmt == &df_dwarf) {
             /* for dwarf debugging information, create the ten dwarf sections */
 
             /* this function call creates the dwarf sections in memory */
@@ -1247,7 +1237,7 @@ static struct SAA *elf_build_symtab(int32_t *len, int32_t *local)
       * dwarf needs symbols for debug sections
       * which are relocation targets.
       */
-     if (of_elf64.current_dfmt == &df_dwarf) {
+     if (dfmt == &df_dwarf) {
         dwarf_infosym = *local;
         p = entry;
         WRITELONG(p, 0);        /* no symbol name */
@@ -1433,8 +1423,8 @@ static struct dfmt df_dwarf = {
     "dwarf",
     dwarf64_init,
     dwarf64_linenum,
-    debug64_deflabel,
-    debug64_directive,
+    null_debug_deflabel,
+    null_debug_directive,
     debug64_typevalue,
     dwarf64_output,
     dwarf64_cleanup
@@ -1444,8 +1434,8 @@ static struct dfmt df_stabs = {
     "stabs",
     null_debug_init,
     stabs64_linenum,
-    debug64_deflabel,
-    debug64_directive,
+    null_debug_deflabel,
+    null_debug_directive,
     debug64_typevalue,
     stabs64_output,
     stabs64_cleanup
@@ -1457,6 +1447,7 @@ struct ofmt of_elf64 = {
     "ELF64 (x86_64) object files (e.g. Linux)",
     "elf64",
     0,
+    64,
     elf64_debugs_arr,
     &df_stabs,
     elf_stdmac,
@@ -1473,22 +1464,6 @@ struct ofmt of_elf64 = {
 };
 
 /* common debugging routines */
-static void debug64_deflabel(char *name, int32_t segment, int64_t offset,
-                             int is_global, char *special)
-{
-    (void)name;
-    (void)segment;
-    (void)offset;
-    (void)is_global;
-    (void)special;
-}
-
-static void debug64_directive(const char *directive, const char *params)
-{
-    (void)directive;
-    (void)params;
-}
-
 static void debug64_typevalue(int32_t type)
 {
     int32_t stype, ssize;
@@ -1682,7 +1657,7 @@ static void stabs64_generate(void)
          * the source-file, the n_desc field should be set to the number
          * of remaining stabs
          */
-        WRITE_STAB(sptr, fileidx[0], 0, 0, 0, strlen(allfiles[0] + 12));
+        WRITE_STAB(sptr, fileidx[0], 0, 0, 0, stabstrlen);
 
         /* this is the stab for the main source file */
         WRITE_STAB(sptr, fileidx[mainfileindex], N_SO, 0, 0, 0);
