@@ -1,6 +1,6 @@
 /* ----------------------------------------------------------------------- *
  *
- *   Copyright 1996-2013 The NASM Authors - All Rights Reserved
+ *   Copyright 1996-2016 The NASM Authors - All Rights Reserved
  *   See the file AUTHORS included with the NASM distribution for
  *   the specific copyright holders.
  *
@@ -160,8 +160,6 @@ static struct elf_symbol *lastsym;
 
 /* common debugging routines */
 static void debugx32_typevalue(int32_t);
-static void debugx32_deflabel(char *, int32_t, int64_t, int, char *);
-static void debugx32_directive(const char *, const char *);
 
 /* stabs debugging routines */
 static void stabsx32_linenum(const char *filename, int32_t linenumber, int32_t);
@@ -190,7 +188,6 @@ static int32_t elf_gottpoff_sect;
 
 static void elf_init(void)
 {
-    maxbits = 64;
     sects = NULL;
     nsects = sectlen = 0;
     syms = saa_init((int32_t)sizeof(struct elf_symbol));
@@ -223,13 +220,11 @@ static void elf_init(void)
 
 }
 
-static void elf_cleanup(int debuginfo)
+static void elf_cleanup(void)
 {
     struct elf_reloc *r;
     int i;
 
-    (void)debuginfo;
-
     elf_write();
     for (i = 0; i < nsects; i++) {
         if (sects[i]->type != SHT_NOBITS)
@@ -246,9 +241,7 @@ static void elf_cleanup(int debuginfo)
     saa_free(syms);
     raa_free(bsym);
     saa_free(strs);
-    if (of_elfx32.current_dfmt) {
-        of_elfx32.current_dfmt->cleanup();
-    }
+    dfmt->cleanup();
 }
 
 /* add entry to the elf .shstrtab section */
@@ -388,7 +381,7 @@ static void elf_deflabel(char *name, int32_t segment, int64_t offset,
                 stdscan_reset();
                 stdscan_set(p);
                 tokval.t_type = TOKEN_INVALID;
-                e = evaluate(stdscan, NULL, &tokval, NULL, 1, nasm_error, NULL);
+                e = evaluate(stdscan, NULL, &tokval, NULL, 1, NULL);
                 if (e) {
                     if (!is_simple(e))
                         nasm_error(ERR_NONFATAL, "cannot use relocatable"
@@ -427,7 +420,7 @@ static void elf_deflabel(char *name, int32_t segment, int64_t offset,
             /* we have to be sure at least text section is there */
             int tempint;
             if (segment != elf_section_names(".text", 2, &tempint))
-                nasm_error(ERR_PANIC, "strange segment conditions in ELF driver");
+                nasm_panic(0, "strange segment conditions in ELF driver");
         }
         for (i = 0; i < nsects; i++) {
             if (segment == sects[i]->index) {
@@ -530,8 +523,7 @@ static void elf_deflabel(char *name, int32_t segment, int64_t offset,
                     stdscan_reset();
                     stdscan_set(special + n);
                     tokval.t_type = TOKEN_INVALID;
-                    e = evaluate(stdscan, NULL, &tokval, &fwd, 0, nasm_error,
-                                 NULL);
+                    e = evaluate(stdscan, NULL, &tokval, &fwd, 0, NULL);
                     if (fwd) {
                         sym->nextfwd = fwds;
                         fwds = sym;
@@ -702,7 +694,7 @@ static void elf_out(int32_t segto, const void *data,
     if (!s) {
         int tempint;            /* ignored */
         if (segto != elf_section_names(".text", 2, &tempint))
-            nasm_error(ERR_PANIC, "strange segment conditions in ELF driver");
+            nasm_panic(0, "strange segment conditions in ELF driver");
         else {
             s = sects[nsects - 1];
             i = nsects - 1;
@@ -710,13 +702,11 @@ static void elf_out(int32_t segto, const void *data,
     }
 
     /* again some stabs debugging stuff */
-    if (of_elfx32.current_dfmt) {
-        sinfo.offset = s->len;
-        sinfo.section = i;
-        sinfo.segto = segto;
-        sinfo.name = s->name;
-        of_elfx32.current_dfmt->debug_output(TY_DEBUGSYMLIN, &sinfo);
-    }
+    sinfo.offset = s->len;
+    sinfo.section = i;
+    sinfo.segto = segto;
+    sinfo.name = s->name;
+    dfmt->debug_output(TY_DEBUGSYMLIN, &sinfo);
     /* end of debugging stuff */
 
     if (s->type == SHT_NOBITS && type != OUT_RESERVE) {
@@ -738,14 +728,14 @@ static void elf_out(int32_t segto, const void *data,
 
     case OUT_RAWDATA:
         if (segment != NO_SEG)
-            nasm_error(ERR_PANIC, "OUT_RAWDATA with other than NO_SEG");
+            nasm_panic(0, "OUT_RAWDATA with other than NO_SEG");
         elf_sect_write(s, data, size);
 	break;
 
     case OUT_ADDRESS:
     {
         int isize = (int)size;
-        int asize = abs(size);
+        int asize = abs((int)size);
 
         addr = *(int64_t *)data;
         if (segment == NO_SEG) {
@@ -775,7 +765,7 @@ static void elf_out(int32_t segto, const void *data,
                     elf_add_reloc(s, segment, addr, R_X86_64_64);
                     break;
                 default:
-                    nasm_error(ERR_PANIC, "internal error elfx32-hpa-871");
+                    nasm_panic(0, "internal error elfx32-hpa-871");
                     break;
                 }
                 addr = 0;
@@ -833,7 +823,7 @@ static void elf_out(int32_t segto, const void *data,
                     addr = 0;
                     break;
                 default:
-                    nasm_error(ERR_PANIC, "internal error elfx32-hpa-903");
+                    nasm_panic(0, "internal error elfx32-hpa-903");
                     break;
                 }
             } else if (wrt == elf_plt_sect + 1) {
@@ -861,7 +851,7 @@ static void elf_out(int32_t segto, const void *data,
     rel12adr:
         addr = *(int64_t *)data - size;
         if (segment == segto)
-            nasm_error(ERR_PANIC, "intra-segment OUT_REL1ADR");
+            nasm_panic(0, "intra-segment OUT_REL1ADR");
         if (segment == NO_SEG) {
             /* Do nothing */
         } else if (segment % 2) {
@@ -882,7 +872,7 @@ static void elf_out(int32_t segto, const void *data,
     case OUT_REL4ADR:
         addr = *(int64_t *)data - size;
         if (segment == segto)
-            nasm_error(ERR_PANIC, "intra-segment OUT_REL4ADR");
+            nasm_panic(0, "intra-segment OUT_REL4ADR");
         if (segment == NO_SEG) {
             /* Do nothing */
         } else if (segment % 2) {
@@ -941,9 +931,9 @@ static void elf_write(void)
      * relocation sections for the user sections.
      */
     nsections = sec_numspecial + 1;
-    if (of_elfx32.current_dfmt == &df_stabs)
+    if (dfmt == &df_stabs)
         nsections += 3;
-    else if (of_elfx32.current_dfmt == &df_dwarf)
+    else if (dfmt == &df_dwarf)
         nsections += 10;
 
     add_sectname("", ".shstrtab");
@@ -957,14 +947,14 @@ static void elf_write(void)
         }
     }
 
-    if (of_elfx32.current_dfmt == &df_stabs) {
+    if (dfmt == &df_stabs) {
         /* in case the debug information is wanted, just add these three sections... */
         add_sectname("", ".stab");
         add_sectname("", ".stabstr");
         add_sectname(".rel", ".stab");
     }
 
-    else if (of_elfx32.current_dfmt == &df_dwarf) {
+    else if (dfmt == &df_dwarf) {
         /* the dwarf debug standard specifies the following ten sections,
            not all of which are currently implemented,
            although all of them are defined. */
@@ -1061,7 +1051,7 @@ static void elf_write(void)
             p += strlen(p) + 1;
         }
 
-    if (of_elfx32.current_dfmt == &df_stabs) {
+    if (dfmt == &df_stabs) {
         /* for debugging information, create the last three sections
            which are the .stab , .stabstr and .rel.stab sections respectively */
 
@@ -1082,7 +1072,7 @@ static void elf_write(void)
                                stabrellen, sec_symtab, sec_stab, 4, 8);
             p += strlen(p) + 1;
         }
-    } else if (of_elfx32.current_dfmt == &df_dwarf) {
+    } else if (dfmt == &df_dwarf) {
             /* for dwarf debugging information, create the ten dwarf sections */
 
             /* this function call creates the dwarf sections in memory */
@@ -1208,7 +1198,7 @@ static struct SAA *elf_build_symtab(int32_t *len, int32_t *local)
       * dwarf needs symbols for debug sections
       * which are relocation targets.
       */
-     if (of_elfx32.current_dfmt == &df_dwarf) {
+     if (dfmt == &df_dwarf) {
         dwarf_infosym = *local;
         p = entry;
         WRITELONG(p, 0);        /* no symbol name */
@@ -1392,8 +1382,8 @@ static struct dfmt df_dwarf = {
     "dwarf",
     dwarfx32_init,
     dwarfx32_linenum,
-    debugx32_deflabel,
-    debugx32_directive,
+    null_debug_deflabel,
+    null_debug_directive,
     debugx32_typevalue,
     dwarfx32_output,
     dwarfx32_cleanup
@@ -1403,8 +1393,8 @@ static struct dfmt df_stabs = {
     "stabs",
     null_debug_init,
     stabsx32_linenum,
-    debugx32_deflabel,
-    debugx32_directive,
+    null_debug_deflabel,
+    null_debug_directive,
     debugx32_typevalue,
     stabsx32_output,
     stabsx32_cleanup
@@ -1416,6 +1406,7 @@ struct ofmt of_elfx32 = {
     "ELFX32 (x86_64) object files (e.g. Linux)",
     "elfx32",
     0,
+    64,
     elfx32_debugs_arr,
     &df_stabs,
     elf_stdmac,
@@ -1432,22 +1423,6 @@ struct ofmt of_elfx32 = {
 };
 
 /* common debugging routines */
-static void debugx32_deflabel(char *name, int32_t segment, int64_t offset,
-                             int is_global, char *special)
-{
-    (void)name;
-    (void)segment;
-    (void)offset;
-    (void)is_global;
-    (void)special;
-}
-
-static void debugx32_directive(const char *directive, const char *params)
-{
-    (void)directive;
-    (void)params;
-}
-
 static void debugx32_typevalue(int32_t type)
 {
     int32_t stype, ssize;
@@ -1641,7 +1616,7 @@ static void stabsx32_generate(void)
          * the source-file, the n_desc field should be set to the number
          * of remaining stabs
          */
-        WRITE_STAB(sptr, fileidx[0], 0, 0, 0, strlen(allfiles[0] + 12));
+        WRITE_STAB(sptr, fileidx[0], 0, 0, 0, stabstrlen);
 
         /* this is the stab for the main source file */
         WRITE_STAB(sptr, fileidx[mainfileindex], N_SO, 0, 0, 0);
