@@ -1,6 +1,6 @@
 /* ----------------------------------------------------------------------- *
  *   
- *   Copyright 1996-2013 The NASM Authors - All Rights Reserved
+ *   Copyright 1996-2016 The NASM Authors - All Rights Reserved
  *   See the file AUTHORS included with the NASM distribution for
  *   the specific copyright holders.
  *
@@ -185,13 +185,14 @@ static int32_t ieee_entry_seg, ieee_entry_ofs;
 static int checksum;
 
 extern struct ofmt of_ieee;
+static struct dfmt ladsoft_debug_form;
 
 static void ieee_data_new(struct ieeeSection *);
 static void ieee_write_fixup(int32_t, int32_t, struct ieeeSection *,
                              int, uint64_t, int32_t);
 static void ieee_install_fixup(struct ieeeSection *, struct ieeeFixupp *);
 static int32_t ieee_segment(char *, int, int *);
-static void ieee_write_file(int debuginfo);
+static void ieee_write_file(void);
 static void ieee_write_byte(struct ieeeSection *, int);
 static void ieee_write_word(struct ieeeSection *, int);
 static void ieee_write_dword(struct ieeeSection *, int32_t);
@@ -232,10 +233,10 @@ static int ieee_set_info(enum geninfo type, char **val)
 /*
  * Rundown
  */
-static void ieee_cleanup(int debuginfo)
+static void ieee_cleanup(void)
 {
-    ieee_write_file(debuginfo);
-    of_ieee.current_dfmt->cleanup();
+    ieee_write_file();
+    dfmt->cleanup();
     while (seghead) {
         struct ieeeSection *segtmp = seghead;
         seghead = seghead->next;
@@ -305,7 +306,7 @@ static void ieee_deflabel(char *name, int32_t segment,
      * First check for the double-period, signifying something
      * unusual.
      */
-    if (name[0] == '.' && name[1] == '.') {
+    if (name[0] == '.' && name[1] == '.' && name[2] != '@') {
         if (!strcmp(name, "..start")) {
             ieee_entry_seg = segment;
             ieee_entry_ofs = offset;
@@ -419,7 +420,7 @@ static void ieee_out(int32_t segto, const void *data,
     if (!any_segs) {
         int tempint;            /* ignored */
         if (segto != ieee_segment("__NASMDEFSEG", 2, &tempint))
-            nasm_error(ERR_PANIC, "strange segment conditions in IEEE driver");
+            nasm_panic(0, "strange segment conditions in IEEE driver");
     }
 
     /*
@@ -429,7 +430,7 @@ static void ieee_out(int32_t segto, const void *data,
         if (seg->index == segto)
             break;
     if (!seg)
-        nasm_error(ERR_PANIC, "code directed to nonexistent segment?");
+        nasm_panic(0, "code directed to nonexistent segment?");
 
     if (type == OUT_RAWDATA) {
         ucdata = data;
@@ -438,7 +439,7 @@ static void ieee_out(int32_t segto, const void *data,
     } else if (type == OUT_ADDRESS || type == OUT_REL2ADR ||
                type == OUT_REL4ADR) {
         if (type == OUT_ADDRESS)
-            size = abs(size);
+            size = abs((int)size);
         else if (segment == NO_SEG)
             nasm_error(ERR_NONFATAL, "relative call to absolute address not"
                   " supported by IEEE format");
@@ -535,7 +536,7 @@ static void ieee_write_fixup(int32_t segment, int32_t wrt,
                     }
 
                 } else
-                    nasm_error(ERR_PANIC,
+                    nasm_panic(0,
                           "unrecognised WRT value in ieee_write_fixup");
             } else
                 nasm_error(ERR_NONFATAL, "target of WRT must be a section ");
@@ -577,7 +578,7 @@ static void ieee_write_fixup(int32_t segment, int32_t wrt,
                  */
                 if (eb) {
                     if (realtype == OUT_REL2ADR || realtype == OUT_REL4ADR) {
-                        nasm_error(ERR_PANIC,
+                        nasm_panic(0,
                               "Segment of a rel not supported in ieee_write_fixup");
                     } else {
                         /* If we want the segment */
@@ -588,7 +589,7 @@ static void ieee_write_fixup(int32_t segment, int32_t wrt,
 
                 } else
                     /* If we get here the seg value doesn't make sense */
-                    nasm_error(ERR_PANIC,
+                    nasm_panic(0,
                           "unrecognised segment value in ieee_write_fixup");
             }
 
@@ -643,7 +644,7 @@ static void ieee_write_fixup(int32_t segment, int32_t wrt,
 
                 } else
                     /* If we get here the seg value doesn't make sense */
-                    nasm_error(ERR_PANIC,
+                    nasm_panic(0,
                           "unrecognised segment value in ieee_write_fixup");
             }
         }
@@ -899,7 +900,7 @@ static void ieee_filename(char *inname, char *outname)
     standard_extension(inname, outname, ".o");
 }
 
-static void ieee_write_file(int debuginfo)
+static void ieee_write_file(void)
 {
     struct tm *thetime;
     time_t reltime;
@@ -911,6 +912,7 @@ static void ieee_write_file(int debuginfo)
     struct ieeeFixupp *fix;
     struct Array *arr;
     int i;
+    const bool debuginfo = (dfmt == &ladsoft_debug_form);
 
     /*
      * Write the module header
@@ -994,7 +996,7 @@ static void ieee_write_file(int debuginfo)
             if (seg->index == ieee_entry_seg)
                 break;
         if (!seg)
-            nasm_error(ERR_PANIC, "Start address records are incorrect");
+            nasm_panic(0, "Start address records are incorrect");
         else
             ieee_putascii("ASG,R%X,%lX,+.\n", seg->ieee_index,
                           ieee_entry_ofs);
@@ -1357,7 +1359,7 @@ static void dbgls_linnum(const char *lnfname, int32_t lineno, int32_t segto)
     if (!any_segs) {
         int tempint;            /* ignored */
         if (segto != ieee_segment("__NASMDEFSEG", 2, &tempint))
-            nasm_error(ERR_PANIC, "strange segment conditions in OBJ driver");
+            nasm_panic(0, "strange segment conditions in OBJ driver");
     }
 
     /*
@@ -1367,7 +1369,7 @@ static void dbgls_linnum(const char *lnfname, int32_t lineno, int32_t segto)
         if (seg->index == segto)
             break;
     if (!seg)
-        nasm_error(ERR_PANIC, "lineno directed to nonexistent segment?");
+        nasm_panic(0, "lineno directed to nonexistent segment?");
 
     for (fn = fnhead; fn; fn = fn->next) {
         if (!nasm_stricmp(lnfname, fn->name))
@@ -1397,20 +1399,16 @@ static void dbgls_deflabel(char *name, int32_t segment,
     /* Keep compiler from warning about special */
     (void)special;
 
+    /*
+     * Note: ..[^@] special symbols are filtered in labels.c
+     */
+
     /*
      * If it's a special-retry from pass two, discard it.
      */
     if (is_global == 3)
         return;
 
-    /*
-     * First check for the double-period, signifying something
-     * unusual.
-     */
-    if (name[0] == '.' && name[1] == '.' && name[2] != '@') {
-        return;
-    }
-
     /*
      * Case (i):
      */
@@ -1517,6 +1515,7 @@ struct ofmt of_ieee = {
     "IEEE-695 (LADsoft variant) object file format",
     "ieee",
     OFMT_TEXT,
+    32,
     ladsoft_debug_arr,
     &ladsoft_debug_form,
     NULL,
